Package: atSNP
Type: Package
Title: Affinity test for identifying regulatory SNPs
Version: 1.29.0
Date: 2021-4-26
Authors@R: c(person(given = "Chandler", family = "Zuo", role = "aut"),
        person(given = "Sunyoung",family = "Shin",role = c("aut", "cre"), email = "sunyoung.shin@utdallas.edu"),
        person(given = "Sunduz", family = "Keles", role = "aut"))
Description: atSNP performs affinity tests of motif matches with the SNP or the reference genomes and SNP-led changes in motif matches.
License: GPL-2
LinkingTo: Rcpp
Depends: R (>= 3.6)
Suggests: BiocStyle, knitr, rmarkdown
Imports: BSgenome, BiocFileCache, BiocParallel, Rcpp, data.table,
        ggplot2, grDevices, graphics, grid, motifStack, rappdirs,
        stats, testthat, utils, lifecycle
RoxygenNote: 6.1.1
biocViews: Software, ChIPSeq, GenomeAnnotation, MotifAnnotation,
        Visualization
URL: https://github.com/sunyoungshin/atSNP
BugReports: https://github.com/sunyoungshin/atSNP/issues
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/atSNP
git_branch: devel
git_last_commit: 7995f08
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-28 21:18:16 UTC; biocbuild
Author: Chandler Zuo [aut],
  Sunyoung Shin [aut, cre],
  Sunduz Keles [aut]
Maintainer: Sunyoung Shin <sunyoung.shin@utdallas.edu>
