# Generated by roxygen2: do not edit by hand

export(ComputeMotifScore)
export(ComputePValues)
export(GetIUPACSequence)
export(LoadFastaData)
export(LoadMotifLibrary)
export(LoadSNPData)
export(MatchSubsequence)
export(dtMotifMatch)
export(plotMotifMatch)
import(BSgenome)
import(BiocFileCache)
import(Rcpp)
import(data.table)
import(ggplot2)
import(grid)
import(rappdirs)
import(lifecycle)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,SnowParam)
importFrom(BiocParallel,bpmapply)
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(motifStack,pcm2pfm)
importFrom(motifStack,plotMotifLogo)
importFrom(stats,quantile)
importFrom(stats,var)
importFrom(utils,data)
importFrom(utils,read.table)
importFrom(utils,write.table)
useDynLib(atSNP)
